package com.gaitup.app.sample;


import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.gaitup.lib.physilog.AvailblePhysilogListener;
import com.gaitup.lib.physilog.BluetoothUnavailableException;
import com.gaitup.lib.physilog.ConnectionState;
import com.gaitup.lib.physilog.Physilog;
import com.gaitup.lib.physilog.PhysilogManager;

import java.util.List;


/**
 * This fragment is an example on how to control the connection / disconnection of physilogs manually
 * without using AvailablePhysilogsFragment.
 */
public class ConnectionFragment extends Fragment {

    private TextView demoText;
    private View rootView;
    private FloatingActionButton fab;

    private PhysilogManager physilogManager;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        rootView = inflater.inflate(R.layout.fragment_connection, container, false);
        demoText = (TextView) rootView.findViewById(R.id.demoText);
        demoText.setText("");
        fab = (FloatingActionButton) rootView.findViewById(R.id.floatingActionButton);
        fab.setOnClickListener(fabListener);
        return rootView;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //Get the instance of PhysilogManager and check it's needed permissions.
        try {
            physilogManager = PhysilogManager.getInstance(getContext());
            physilogManager.checkAndRequestPermissions(getActivity());
        } catch (BluetoothUnavailableException e) {
            Toast.makeText(getContext(), getString(com.gaitup.lib.physilog.R.string.ble_not_supported), Toast.LENGTH_SHORT);
        }

        //Register for the wanted broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());
        bm.registerReceiver(broadcastReceiverNewPhysilogConnected, new IntentFilter("new_physilog_connected"));
        bm.registerReceiver(broadcastReceiverPhysilogDisconnected, new IntentFilter("physilog_disconnected"));

        physilogManager.initActivity();

    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        //Unregister for the broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());
        bm.unregisterReceiver(broadcastReceiverNewPhysilogConnected);
        bm.unregisterReceiver(broadcastReceiverPhysilogDisconnected);

        //safe-guard in case of early termination
        physilogManager.stopAllStreams();
    }

    @Override
    public void onResume() {
        super.onResume();
        //Refresh the connection status with all the physilogs and re-connnect
        //with the one who lost connection during the application's pause
        physilogManager.autoConnectToPhysilog("PHY5_293");
        physilogManager.autoConnectToPhysilog("PHY5_304");
        physilogManager.autoConnectToPhysilog("PHY5_293");
        physilogManager.autoConnectToPhysilog("PHY5_304");
    }

    //Handle clicks on the floating action button
    private View.OnClickListener fabListener = new View.OnClickListener() {
        @Override
        public void onClick(View view){
        demoText.append("Start Search for available Physilogs\n");
        physilogManager.scanForPhysilogs(physilogListener);
        }
    };

    //Called when the scan for Physilogs has finished
    private AvailblePhysilogListener physilogListener = new AvailblePhysilogListener() {
        @Override
        public void availablePhysilogs(List<Physilog> physilogs) {
            demoText.append("Found those:\n");
            for(Physilog ph : physilogs){
                demoText.append("* " + ph.getName() + "\n");
            }

            //Connect to the first one in the list:
            if(physilogs.size() > 0){
                demoText.append("Try connecting to " + physilogs.get(0).getName());
                physilogManager.connectToPhysilog(physilogs.get(0).getDevice());
            }
        }
    };

    BroadcastReceiver broadcastReceiverNewPhysilogConnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            demoText.append("New physilog Connected: " +  device.getName() + "\n");
        }
    };

    BroadcastReceiver broadcastReceiverPhysilogDisconnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            demoText.append("Physilog Disconnected: " +  device.getName() + "\n");

        }
    };
}
