package com.gaitup.app.sample;

import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Environment;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.gaitup.lib.physilog.AvailblePhysilogListener;
import com.gaitup.lib.physilog.BluetoothUnavailableException;
import com.gaitup.lib.physilog.Physilog;
import com.gaitup.lib.physilog.PhysilogManager;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/** 
 * Example on how to have the physilog's measurements be
 * recorded in a file transferred on the android device.
 */
public class FileFragment extends Fragment {

    //Global state of the fragment
    enum State {NotReady, Ready, Recording};

    private View rootView;
    private FloatingActionButton fbToggle;

    private PhysilogManager physilogManager;
    private List<BluetoothDevice> devices;


    private ListView deviceList;
    private ArrayList<DeviceStatus> deviceStatus;
    private BaseAdapter adapter;

    State recordingState;



    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        rootView = inflater.inflate(R.layout.fragment_file, container, false);
        deviceList = (ListView) rootView.findViewById(R.id.listSensors);
        deviceStatus = new ArrayList<>();
        adapter = new DeviceAdapter(getContext(), deviceStatus);
        deviceList.setAdapter(adapter);


        fbToggle = (FloatingActionButton) rootView.findViewById(R.id.fbToggle);
        fbToggle.setVisibility(View.GONE);

        fbToggle.setOnClickListener(fabListener);
        return rootView;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //Get the instance of the PhysilogManager
        Context context = getContext();
        try {
            physilogManager = PhysilogManager.getInstance(context);
            physilogManager.checkAndRequestPermissions(getActivity());
        } catch (BluetoothUnavailableException e) {
            Toast.makeText(context, getString(com.gaitup.lib.physilog.R.string.ble_not_supported), Toast.LENGTH_SHORT);
        }

        recordingState = State.NotReady;
        devices = new ArrayList<>();

        //Choose where the files will be recorded
        String dir = Environment.getExternalStorageDirectory().toString() + getString(R.string.file_folder);
        physilogManager.setFilesDirectory(dir);

        //Register to the needed broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());

        bm.registerReceiver(broadcastReceiverNewPhysilogConnected, new IntentFilter("new_physilog_connected"));
        bm.registerReceiver(broadcastReceiverPhysilogDisconnected, new IntentFilter("physilog_disconnected"));
        bm.registerReceiver(broadcastReceiverStateChanged, new IntentFilter("state_changed"));

        bm.registerReceiver(broadcastReceiverFileStreamProgress, new IntentFilter("file_stream_progress"));
        bm.registerReceiver(broadcastReceiverFileName, new IntentFilter("file_name"));
        bm.registerReceiver(broadcastReceiverFileGenerated, new IntentFilter("file_generated"));
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        //Unregister to the broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());

        bm.unregisterReceiver(broadcastReceiverNewPhysilogConnected);
        bm.unregisterReceiver(broadcastReceiverPhysilogDisconnected);
        bm.unregisterReceiver(broadcastReceiverStateChanged);

        bm.unregisterReceiver(broadcastReceiverFileStreamProgress);
        bm.unregisterReceiver(broadcastReceiverFileName);
        bm.unregisterReceiver(broadcastReceiverFileGenerated);

        //safe-guard in case of early termination
        physilogManager.discardAllFiles();
    }

    @Override
    public void onResume() {
        super.onResume();
        physilogManager.initActivity();
    }










    BroadcastReceiver broadcastReceiverNewPhysilogConnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            if(!devices.contains(device)){
                //New device
                devices.add(device);

                DeviceStatus status = new DeviceStatus();
                status.name = device.getName();
                deviceStatus.add(status);

                //See if every connected physilog is ready
                checkAllReady();
            }

        }
    };




    BroadcastReceiver broadcastReceiverPhysilogDisconnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){

                //remove the disconnected device
                devices.remove(deviceIndex);
                deviceStatus.remove(deviceIndex);
                adapter.notifyDataSetChanged();

                fbToggle.setVisibility(View.GONE);
                checkAllReady();
            }
        }
    };

    BroadcastReceiver broadcastReceiverStateChanged = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            byte state = (byte) b.getByte("new_state");

            //update the displayed state of the physilog
            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1) {

                deviceStatus.get(deviceIndex).state = physilogManager.getPhysilog(device).getStateText();
            }
            checkAllReady();

            //As soon as one of the connected physilog is recording we switch
            //the fragment state to recording.
            if(state == Physilog.STATE_PHYSILOG_ON_FILE){
                recordingState = State.Recording;
            }

            adapter.notifyDataSetChanged();
        }
    };


    // File related broadcasts

    BroadcastReceiver broadcastReceiverFileStreamProgress = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            int progress = b.getInt("progress");

            //Display the newly received progress
            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1) {
                deviceStatus.get(deviceIndex).progress = progress + "%";
            }



            adapter.notifyDataSetChanged();
        }
    };

    BroadcastReceiver broadcastReceiverFileName = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            String name = b.getString("name");

            //Display the received file name
            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1) {
                deviceStatus.get(deviceIndex).path = name;
            }
            adapter.notifyDataSetChanged();
        }
    };

    BroadcastReceiver broadcastReceiverFileGenerated = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            int progress = b.getInt("progress");

            if(recordingState == State.Recording){

                //Display the received progress and replace the file name with
                //the path of the now finalized file
                int deviceIndex = devices.indexOf(device);
                if(deviceIndex != -1) {

                    DeviceStatus status = deviceStatus.get(deviceIndex);
                //status.progress = progress + "%";
                status.path = physilogManager.getPhysilog(device).getFilePath();
                }

                adapter.notifyDataSetChanged();
            }
        }
    };

    private void checkAllReady(){

        boolean allReady = devices.size() > 0 ? true : false;

        for(BluetoothDevice device: devices ){
            if(physilogManager.getPhysilog(device).getState() != Physilog.STATE_STANDBY_OFF) {
                allReady = false;
                break;
            }
        }

        if(allReady){
            //When every connected physilog is ready, we can offer to record
            recordingState = State.Ready;
            fbToggle.setImageResource(R.drawable.ic_play_arrow_white_36dp);
            fbToggle.setVisibility(View.VISIBLE);
        } else if (devices.size()>0){

            boolean allStreaming = true;


            for(BluetoothDevice device: devices ){
                if(physilogManager.getPhysilog(device).getState() != Physilog.STATE_PHYSILOG_ON_FILE) {
                    allStreaming = false;
                    break;
                }
            }
            if(allStreaming) {
                recordingState = State.Recording;
                fbToggle.setImageResource(R.drawable.ic_stop_white_36dp);
                fbToggle.setVisibility(View.VISIBLE);
            } else{
                fbToggle.setVisibility(View.INVISIBLE);

            }
                //When every connected physilog is ready, we can offer to record
;

        }
    }

    //Handle clicks on the recording button
    private View.OnClickListener fabListener = new View.OnClickListener() {
        @Override
        public void onClick(View view){

            boolean requestRecording = true;
            if(recordingState == State.Ready) {

                //Start the recording for every physilog connected
                //physilogManager.prepareForRecording();

                for(int i = 0; i < devices.size(); i++){
                    deviceStatus.get(i).progress = "0%";
                }
                physilogManager.startAllFilesStreaming();

                fbToggle.setImageResource(R.drawable.ic_stop_white_36dp);

            } else if(recordingState == State.Recording){

                physilogManager.stopAllFilesStreaming();

                fbToggle.setVisibility(View.GONE);
            }
        }
    };


    private static class DeviceStatus {
        String name;
        String state;
        String progress;
        String path;

        public DeviceStatus() {
            name = "";
            state = "";
            progress = "";
            path = "";
        }
    }

    public class DeviceAdapter extends ArrayAdapter<DeviceStatus> {
        public DeviceAdapter(Context context, ArrayList<DeviceStatus> devices) {
            super(context, R.layout.demo_file, devices);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            // Get the data item for this position
            DeviceStatus status = getItem(position);
            // Check if an existing view is being reused, otherwise inflate the view
            if (convertView == null) {
                convertView = LayoutInflater.from(getContext()).inflate(R.layout.demo_file, parent, false);
            }

            TextView name = (TextView) convertView.findViewById(R.id.textName);
            TextView state = (TextView) convertView.findViewById(R.id.textState);
            TextView progress = (TextView) convertView.findViewById(R.id.textProgress);
            TextView path = (TextView) convertView.findViewById(R.id.textPath);

            name.setText(status.name);
            state.setText(status.state);
            progress.setText(status.progress);
            path.setText(status.path);

            return convertView;
        }
    }
}
