package com.gaitup.app.sample;

import android.app.ProgressDialog;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Configuration;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.view.ScaleGestureDetectorCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.gaitup.lib.physilog.AvailablePhysilogsFragment;
import com.gaitup.lib.physilog.BluetoothUnavailableException;
import com.gaitup.lib.physilog.Physilog;
import com.gaitup.lib.physilog.PhysilogManager;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements NavigationView.OnNavigationItemSelectedListener {

    private final static String AVAILABLE_PHYSILOGS_TAG = "AvailableFragmentTag";

    //Used in portrait mode to show the list of available physilogs
    private ImageButton showPhysilog;
    private Button showPhysilogDone;

    private Fragment currentFragment;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        showPhysilog = (ImageButton) findViewById(R.id.show_physilog);
        showPhysilog.setOnClickListener(showPhysilogListener);
        showPhysilogDone = (Button) findViewById(R.id.show_physilogs_done);
        showPhysilogDone.setOnClickListener(showPhysilogDoneListener);

        //Only display the buttons for some devices
        checkToggleButtons();

        //Init navigation drawer
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.setDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        //Start with sensor fragment
        changeFragment(SensorFragment.class, navigationView.getMenu().getItem(0).getTitle());
        navigationView.getMenu().getItem(0).setChecked(true);
    }


    @Override
    public void onBackPressed() {
        //Close the navigation drawer if open when back is pressed
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        try{
            //Safe-guard disconnect from everything before closing
            PhysilogManager physilogManager = PhysilogManager.getInstance(this);
            for(Physilog ph: physilogManager.getConnectedPhysilogs()){
                physilogManager.disconnectFromPhysilog(ph.getDevice());
            }
        } catch (BluetoothUnavailableException e){
            e.printStackTrace();
        }
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);

        boolean isLargeorSmall =
                (newConfig.screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE ||
                (newConfig.screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_NORMAL;

        if(currentFragment != null && isLargeorSmall){
            //We need to switch between landscape and portrait for the fragment
            //So we re-create it
            changeFragment(currentFragment.getClass(), getSupportActionBar().getTitle());
            checkToggleButtons();
        }
    }

    public void checkToggleButtons(){
        //only show the buttons in portrait for small/normal devices
        Configuration config = getResources().getConfiguration();

        boolean isLargeorSmall =
                (config.screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_LARGE ||
                (config.screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) == Configuration.SCREENLAYOUT_SIZE_NORMAL;
        boolean isPortrait = config.orientation == Configuration.ORIENTATION_PORTRAIT;

        if(isPortrait && isLargeorSmall){
            showPhysilog.setVisibility(View.VISIBLE);
            showPhysilogDone.setVisibility(View.GONE);
        } else {
            showPhysilog.setVisibility(View.GONE);
            showPhysilogDone.setVisibility(View.GONE);
        }
    }

    @Override
    public boolean onNavigationItemSelected(MenuItem item) {
        // Handle Navigation drawer menu clicks
        int id = item.getItemId();
        Class fragmentClass = null;

        if (id == R.id.sensor) {
            fragmentClass = SensorFragment.class;
        } else if (id == R.id.file) {
            fragmentClass = FileFragment.class;
        } else if (id == R.id.bluetooth) {
            fragmentClass = ConnectionFragment.class;
        } else if (id == R.id.remote) {
                   fragmentClass = RemoteControlFragment.class;
        } else if (id == R.id.file_transfer) {
            fragmentClass = FileTransferFragment.class;
        }

        changeFragment(fragmentClass, item.getTitle());

        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    public void changeFragment(Class fragmentClass, CharSequence title){
        try {
            currentFragment = (Fragment) fragmentClass.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }

        FragmentManager fragmentManager = getSupportFragmentManager();
        fragmentManager.beginTransaction().replace(R.id.flContent, currentFragment).commit();

        ActionBar ab = getSupportActionBar();
        ab.setTitle(title);

        checkToggleButtons();
    }

    private View.OnClickListener showPhysilogListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {

            //Add the AvailablePhysilogsFragment to the screen to let the user do the connections.
            FragmentManager fragmentManager = getSupportFragmentManager();
            Fragment frag = fragmentManager.findFragmentByTag(AVAILABLE_PHYSILOGS_TAG);

            if(frag == null){
                fragmentManager.beginTransaction()
                        .setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
                        .add(R.id.flContent, new AvailablePhysilogsFragment(), AVAILABLE_PHYSILOGS_TAG)
                        .commit();

            } else {
                fragmentManager.beginTransaction()
                        .setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
                        .show(frag)
                        .commit();
            }
            showPhysilog.setVisibility(View.GONE);
            showPhysilogDone.setVisibility(View.VISIBLE);
        }
    };

    private View.OnClickListener showPhysilogDoneListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {

            //Hide the AvailablePhysilogsFragment
            FragmentManager fm = getSupportFragmentManager();
            Fragment frag = fm.findFragmentByTag(AVAILABLE_PHYSILOGS_TAG);

            if(frag != null){
                fm.beginTransaction()
                        .setCustomAnimations(android.R.anim.fade_in, android.R.anim.fade_out)
                        .hide(frag)
                        .commit();
            }
            showPhysilog.setVisibility(View.VISIBLE);
            showPhysilogDone.setVisibility(View.GONE);
        }
    };

}
