package com.gaitup.app.sample;

import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.gaitup.lib.physilog.BluetoothUnavailableException;
import com.gaitup.lib.physilog.Physilog;
import com.gaitup.lib.physilog.PhysilogManager;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by gaitup4 on 05.10.2017.
 */

public class RemoteControlFragment extends Fragment {

    //Global state of the fragment
    enum State {NotReady, Ready, Recording};

    private View rootView;
    private FloatingActionButton fbToggle;

    private PhysilogManager physilogManager;
    private List<BluetoothDevice> devices;

    private ListView deviceList;
    private ArrayList<DeviceStatus> deviceStatus;
    private BaseAdapter adapter;

    State recordingState;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        rootView = inflater.inflate(R.layout.fragment_remote_control, container, false);
        deviceList = (ListView) rootView.findViewById(R.id.listSensorsRemote);
        deviceStatus = new ArrayList<>();
        adapter = new DeviceAdapter(getContext(), deviceStatus);
        deviceList.setAdapter(adapter);


        fbToggle = (FloatingActionButton) rootView.findViewById(R.id.fbToggleRemote);
        fbToggle.setVisibility(View.GONE);
        rootView.findViewById(R.id.action_0).setOnClickListener(btnListener);
        rootView.findViewById(R.id.action_1).setOnClickListener(btnListener);
        rootView.findViewById(R.id.action_2).setOnClickListener(btnListener);
        rootView.findViewById(R.id.action_3).setOnClickListener(btnListener);
        rootView.findViewById(R.id.action_4).setOnClickListener(btnListener);
        rootView.findViewById(R.id.action_5).setOnClickListener(btnListener);

        fbToggle.setOnClickListener(fabListener);
        return rootView;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        //Get the instance of the PhysilogManager
        Context context = getContext();
        try {
            physilogManager = PhysilogManager.getInstance(context);
            physilogManager.checkAndRequestPermissions(getActivity());
        } catch (BluetoothUnavailableException e) {
            Toast.makeText(context, getString(com.gaitup.lib.physilog.R.string.ble_not_supported), Toast.LENGTH_SHORT);
        }

        recordingState = State.NotReady;
        devices = new ArrayList<>();

        //Register to the needed broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());

        bm.registerReceiver(broadcastReceiverNewPhysilogConnected, new IntentFilter("new_physilog_connected"));
        bm.registerReceiver(broadcastReceiverPhysilogDisconnected, new IntentFilter("physilog_disconnected"));
        bm.registerReceiver(broadcastReceiverStateChanged, new IntentFilter("state_changed"));
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        //safe-guard in case of early termination
        physilogManager.stopAllRemoteFiles();
        //Unregister to the broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());

        bm.unregisterReceiver(broadcastReceiverNewPhysilogConnected);
        bm.unregisterReceiver(broadcastReceiverPhysilogDisconnected);
        bm.unregisterReceiver(broadcastReceiverStateChanged);

    }

    @Override
    public void onResume() {
        super.onResume();
        physilogManager.initActivity();
    }

    BroadcastReceiver broadcastReceiverNewPhysilogConnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            if(!devices.contains(device)){
                //New device
                devices.add(device);

                DeviceStatus status = new DeviceStatus();
                status.name = device.getName();
                deviceStatus.add(status);

                //See if every connected physilog is ready
                checkAllReady();
            }

        }
    };


    private View.OnClickListener btnListener = new View.OnClickListener()
    {

        public void onClick(View v)
        {
            byte id = (byte)0x00;
            if(v.getId() == R.id.action_0){
                id = (byte)0x00;
            } else if(v.getId() == R.id.action_1){
                id =  (byte)0x01;
            } else if(v.getId() == R.id.action_2){
                id =  (byte)0x02;
            } else if(v.getId() == R.id.action_3){
                id =  (byte)0x03;
            } else if(v.getId() == R.id.action_4){
                id =  (byte)0x04;
            } else if(v.getId() == R.id.action_5){
                id =  (byte)0x05;
            }
            Log.d("btnListener", " action clicked " + id);
            physilogManager.sendEventToAllPhysilogs(id);

        }

    };


    BroadcastReceiver broadcastReceiverPhysilogDisconnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){

                //remove the disconnected device
                devices.remove(deviceIndex);
                deviceStatus.remove(deviceIndex);
                adapter.notifyDataSetChanged();

                fbToggle.setVisibility(View.GONE);
                checkAllReady();
            }
        }
    };

    BroadcastReceiver broadcastReceiverStateChanged = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            byte state = (byte) b.getByte("new_state");

            //update the displayed state of the physilog
            int deviceIndex = devices.indexOf(device);
            deviceStatus.get(deviceIndex).state = physilogManager.getPhysilog(device).getStateText();

            checkAllReady();

            //As soon as one of the connected physilog is recording we switch
            //the fragment state to recording.
            if(state == Physilog.STATE_PHYSILOG_ON_FILE){
                recordingState = State.Recording;
            }

            adapter.notifyDataSetChanged();
        }
    };

    private void checkAllReady(){

        boolean allReady = devices.size() > 0 ? true : false;

        for(BluetoothDevice device: devices ){
            if(physilogManager.getPhysilog(device).getState() != Physilog.STATE_STANDBY_OFF) {
                allReady = false;
                break;
            }
        }


        if(allReady){
            //When every connected physilog is ready, we can offer to record
            recordingState = State.Ready;
            fbToggle.setImageResource(R.drawable.ic_play_arrow_white_36dp);
            fbToggle.setVisibility(View.VISIBLE);
        }else if (devices.size()>0){
            //When every connected physilog is ready, we can offer to record
            recordingState = State.Recording;
            fbToggle.setImageResource(R.drawable.ic_stop_white_36dp);
            fbToggle.setVisibility(View.VISIBLE);

        }
    }

    //Handle clicks on the recording button
    private View.OnClickListener fabListener = new View.OnClickListener() {
        @Override
        public void onClick(View view){

            boolean requiredRecording = false;

            if(recordingState == State.Ready) {

                physilogManager.startAllRemoteFiles();

                fbToggle.setImageResource(R.drawable.ic_stop_white_36dp);

            } else if(recordingState == State.Recording){

                //Stop the recording for every physilog connected
                physilogManager.stopAllRemoteFiles();

                fbToggle.setVisibility(View.GONE);
            }
        }
    };


    private static class DeviceStatus {
        String name;
        String state;

        public DeviceStatus() {
            name = "";
            state = "";
        }
    }

    public class DeviceAdapter extends ArrayAdapter<DeviceStatus> {
        public DeviceAdapter(Context context, ArrayList<DeviceStatus> devices) {
            super(context, R.layout.demo_remote_control, devices);
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            // Get the data item for this position
            DeviceStatus status = getItem(position);
            // Check if an existing view is being reused, otherwise inflate the view
            if (convertView == null) {
                convertView = LayoutInflater.from(getContext()).inflate(R.layout.demo_remote_control, parent, false);
            }

            TextView name = (TextView) convertView.findViewById(R.id.textNameRemote);
            TextView state = (TextView) convertView.findViewById(R.id.textStateRemote);

            name.setText(status.name);
            state.setText(status.state);

            return convertView;
        }
    }
}