package com.gaitup.app.sample;


import android.app.ProgressDialog;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.widget.LinearLayoutCompat;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.gaitup.lib.physilog.BluetoothUnavailableException;
import com.gaitup.lib.physilog.PhysilogManager;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Example on how to access and display the raw values of the physilog
 */
public class SensorFragment extends Fragment {

    private final static long REFRESH_INTERVAL = 32;

    private PhysilogManager physilogManager;
    private ProgressDialog pDialogServer;
    private List<BluetoothDevice> devices;
    private List<float[]> devicesSensors;
    private List<int[]> devicesSettings;
    private List<String> devicesFirmwareVersion;

    private List<float[]> devicesQuat;

    private View rootView;
    private ListView sensorsList;
    private ArrayList<String> sensorsTexts;
    private ArrayAdapter<String> adapter;

    private long last_refresh_time;

    private int acc_freq  = 0  ;
    private int gyro_freq  = 0    ;
    private int baro_freq  = 0    ;
    private int acc_g      = 0   ;
    private int gyro_g     = 0    ;
    private String firmware_version      = "";

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        rootView = inflater.inflate(R.layout.fragment_sensor, container, false);

        sensorsList = (ListView) rootView.findViewById(R.id.listSensors);

        sensorsTexts = new ArrayList<>();
        adapter = new ArrayAdapter<String>(getContext(), R.layout.demo_text, sensorsTexts);
        sensorsList.setAdapter(adapter);

        last_refresh_time = System.currentTimeMillis();

        return rootView;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Context context = getContext();

        //Get the instance of PhysilogManager and check it's needed permissions.
        try {
            physilogManager = PhysilogManager.getInstance(context);
        } catch (BluetoothUnavailableException e){
            Toast.makeText(context, getString(com.gaitup.lib.physilog.R.string.ble_not_supported), Toast.LENGTH_SHORT);
        }
        physilogManager.checkAndRequestPermissions(getActivity());

        pDialogServer = new ProgressDialog(context);
        devices = new ArrayList<>();
        devicesSensors = new ArrayList<>();
        devicesSettings = new ArrayList<>();
        devicesFirmwareVersion = new ArrayList<>();

        devicesQuat = new ArrayList<>();

        //Register for the wanted broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());
        bm.registerReceiver(broadcastReceiverNewPhysilogConnected, new IntentFilter("new_physilog_connected"));
        bm.registerReceiver(broadcastReceiverPhysilogDisconnected, new IntentFilter("physilog_disconnected"));
        bm.registerReceiver(broadcastReceiverNewSensorsData, new IntentFilter("new_sensors_data"));
        bm.registerReceiver(broadcastReceiverNewQuaternionData, new IntentFilter("new_quaternion_data"));
        bm.registerReceiver(broadcastReceiverSensorSettingsValue, new IntentFilter("sensors_settings_value"));
    }

    @Override
    public void onDestroy() {
        super.onDestroy();

        //safe-guard in case of early termination
        physilogManager.stopAllStreams();

        //Unregister for the broadcasts
        LocalBroadcastManager bm = LocalBroadcastManager.getInstance(getContext());
        bm.unregisterReceiver(broadcastReceiverNewPhysilogConnected);
        bm.unregisterReceiver(broadcastReceiverPhysilogDisconnected);
        bm.unregisterReceiver(broadcastReceiverNewSensorsData);
        bm.unregisterReceiver(broadcastReceiverNewQuaternionData);
        bm.unregisterReceiver(broadcastReceiverSensorSettingsValue);
    }

    @Override
    public void onResume() {
        super.onResume();
        physilogManager.initActivity();
    }


    BroadcastReceiver broadcastReceiverNewPhysilogConnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {

            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            if (!devices.contains(device)) {
                //New device
                devices.add(device);

                //8 float values from new_sensors_data and 4 from new_qua
                devicesSensors.add(new float[8]);
                devicesQuat.add(new float[5]);
                devicesSettings.add(new int[5]);
                devicesFirmwareVersion.add(" ");


                sensorsTexts.add("");

                //Little dialog while we receive the settings of the physilog
                pDialogServer.setMessage("Checking the settings of physilog");
                pDialogServer.setCancelable(true);
                pDialogServer.show();

                //Start the stream of information
                physilogManager.getPhysilog(device).startStreaming();
            }
        }
    };

    BroadcastReceiver broadcastReceiverPhysilogDisconnected = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            pDialogServer.dismiss();

            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){
                
                //Remove the device and it's values
                devices.remove(deviceIndex);
                devicesSensors.remove(deviceIndex);
                devicesQuat.remove(deviceIndex);
                sensorsTexts.remove(deviceIndex);

                adapter.notifyDataSetChanged();

                Toast.makeText(getContext(),
                        device.getName() + " " + getResources().getString(R.string.disconnected),
                        Toast.LENGTH_SHORT).show();
            }
        }
    };

    BroadcastReceiver broadcastReceiverNewSensorsData = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            pDialogServer.dismiss();

            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            final float[] values =  b.getFloatArray("SensorsValues");

            //Add data to the list
            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){
                devicesSensors.add(deviceIndex, values);
            }

            //Only refresh the text in the Textview every few milliseconds.
            //Reduce the resources used by the app
            //This is not strictly required but is a good practice in this case where
            //the available information come to quickly for the human eye.
            if (System.currentTimeMillis() - last_refresh_time > REFRESH_INTERVAL){
                refreshScreen();
                last_refresh_time = System.currentTimeMillis();
            }

        }
    };

    BroadcastReceiver broadcastReceiverNewQuaternionData = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Bundle b = intent.getExtras();
            final BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");
            final float[] quat =  b.getFloatArray("QuaternionValues");

            //Add data to the list
            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){
                devicesQuat.add(deviceIndex, quat);
            }

            //Only refresh the text in the Textview every few milliseconds.
            //Reduce the resources used by the app
            //This is not strictly required but is a good practice in this case where
            //the available information come to quickly for the human eye.
            if (System.currentTimeMillis() - last_refresh_time > REFRESH_INTERVAL){
                refreshScreen();
                last_refresh_time = System.currentTimeMillis();
            }
        }
    };

    BroadcastReceiver broadcastReceiverSensorSettingsValue = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            pDialogServer.dismiss();

            Bundle b = intent.getExtras();
            BluetoothDevice device = (BluetoothDevice) b.get("BluetoothDevice");

            //Not used in this sample app, could be used to check that a physilog is properly
            //configured for its task
            int[] settings = new int[5];

            settings[0]   = b.getInt("GyroFreq");
            settings[1]      = b.getInt("GyroG");
            settings[2]    = b.getInt("AccFreq");
            settings[3]       = b.getInt("AccG");
            settings[4]   = b.getInt("BaroFreq");


             firmware_version      = b.getString("firmware_version");


            int deviceIndex = devices.indexOf(device);
            if(deviceIndex != -1){
                devicesSettings.set(deviceIndex, settings);
                devicesFirmwareVersion.set(deviceIndex, firmware_version);
            }

        }
    };

    public void refreshScreen(){
        for (int i = 0; i < devices.size(); i++){

            BluetoothDevice d = devices.get(i);
            float[] sensors = devicesSensors.get(i);
            int[] settings = devicesSettings.get(i);
            String version= devicesFirmwareVersion.get(i);

            float[] quat = devicesQuat.get(i);

            String text = "\n" +
                    d.getName() + "\n" +
                    "Firmware version  " + version + "\n" +
                    "Sensors time:  " + "\n" +
                    "               " + sensors[0] + "\n" +
                    "Gyroscope:     (" + settings[0] + "Hz, "+ settings[1] +"g)" + "\n" +
                    "               " + sensors[1] + "\n" +
                    "               " + sensors[2] + "\n" +
                    "               " + sensors[3] + "\n" +
                    "Accelerometer: (" + settings[2] + "Hz, "+ settings[3] +"g)" + "\n" +
                    "               " + sensors[4] + "\n" +
                    "               " + sensors[5] + "\n" +
                    "               " + sensors[6] + "\n" +
                    "Barometer:     (" + settings[4] + "Hz)" + "\n" +
                    "               " + sensors[7] + "\n" +
                    "Quaternion:    " + "\n" +
                    "               " + quat[0] + "\n" +
                    "               " + quat[1] + "\n" +
                    "               " + quat[2] + "\n" +
                    "               " + quat[3] + "\n" +
                    "Quaternion time:" + "\n" +
                    "               " + quat[4] + "\n";

            sensorsTexts.set(i, text);
        }

        adapter.notifyDataSetChanged();
    }
}
